#!/usr/bin/env python3

"""
AWS Disclaimer.

(c) 2021 Amazon Web Services, Inc. or its affiliates. All Rights Reserved.
This AWS Content is provided subject to the terms of the AWS Customer
Agreement available at https://aws.amazon.com/agreement/ or other written
agreement between Customer and Amazon Web Services, Inc.
"""

import os
import sys
import pandas as pd
import json
import requests
import argparse
from troposphere import Template, config, Ref
    

class ConfigRulesTemplate(Template):
    def __init__(self, ou_name: str, config_rules: list, max_execution_frequency: str):
        self.ou_name = ou_name
        self.config_rules = config_rules
        self.max_execution_frequency = max_execution_frequency
        self.template_name = f"{ou_name.replace(' ', '').strip().lower()}-aws-config-rules.yml"
        self.template_version = "2010-09-09"
        self.template_description = f"AWS Config Rules generated by aws_config_rules.py for OU {ou_name}"
        self.managed_config_rule_s3_url = "https://s3.amazonaws.com/aws-configservice-us-east-1/cloudformation-templates-for-managed-rules/{}.template"
        self.inconsistent_names = {
            "cloudtrail-enabled": "CLOUD_TRAIL_ENABLED",
            "ec2-instance-managed-by-systems-manager": "EC2_INSTANCE_MANAGED_BY_SSM",
            "ec2-instances-in-vpc": "INSTANCES_IN_VPC",
            "multi-region-cloudtrail-enabled": "MULTI_REGION_CLOUD_TRAIL_ENABLED",
            "restricted-common-ports": "RESTRICTED_INCOMING_TRAFFIC",
            "restricted-ssh": "INCOMING_SSH_DISABLED"
        }
        super().__init__()
        self.set_description(self.template_description)
        self.set_version(self.template_version)

    def add_rule_resource(self, rule_name, config_rules_params) -> None:
        """
            Adds config.ConfigRule to Trophosphere cloudformation Template 
        """
        _identifier = self._get_identifier(rule_name)
        _config_rule = config.ConfigRule(
            _namify(rule_name),
            ConfigRuleName=rule_name,
            Source=config.Source(
                    Owner="AWS",
                    SourceIdentifier=_identifier
            )
        )
        _config_rule.__setattr__("InputParameters", self._get_config_rule_params(rule_name, config_rules_params))
        
        config_rule_default_template = requests.get(self.managed_config_rule_s3_url.format(_identifier)).json()
        if "MaximumExecutionFrequency" in config_rule_default_template["Resources"]["AWSConfigRule"]["Properties"].keys():
            _config_rule.__setattr__("MaximumExecutionFrequency", self.max_execution_frequency)
        else:
            _config_rule.__setattr__("Scope", self._get_config_rule_scope(rule_name, config_rules_params,config_rule_default_template["Resources"]["AWSConfigRule"]["Properties"]["Scope"]))
        self.add_resource(_config_rule)

    def _get_config_rule_params(self, rule_name, config_rules_params) -> dict:
        """
            Gets config rule params from config_rules_params if present; uses optional params if KeyError found
        """
        return config_rules_params[rule_name]["InputParameters"] if rule_name in config_rules_params.keys() and "InputParameters" in config_rules_params[rule_name].keys() else {}
        
    @staticmethod
    def _get_config_rule_scope(rule_name, config_rules_params, scope_dict) -> config.Scope:
        """
            Checks for Scope key in parameter file otherwise uses default from AWS defined template
        """
        _scope = config.Scope()
        if rule_name in config_rules_params.keys() and "Scope" in config_rules_params[rule_name].keys():
            _scope.__setattr__("ComplianceResourceTypes", config_rules_params[rule_name]["Scope"]["ComplianceResourceTypes"])
        else:
            _scope.__setattr__("ComplianceResourceTypes", scope_dict["ComplianceResourceTypes"])
        return _scope
        
    def _get_identifier(self, rule_name) -> str:
        """
            Checks for rule_name in inconsistent_names
            Most rules = shield-drt-access -> SHIELD_DRT_ACCESS
            Inconsistent rule example = restricted-ssh -> INCOMING_SSH_DISABLED
        """
        return rule_name.replace("-", "_").upper() if rule_name not in self.inconsistent_names.keys() else self.inconsistent_names[rule_name]
    
def _init_parser() -> argparse.ArgumentParser:
    """
        Initialize command line parser
    """
    _parser = argparse.ArgumentParser()
    _parser.add_argument("--ou", type=str, required=True)
    _parser.add_argument("--config-rule-option", type=str, required=False, default="Implemented", help="Option from Excel sheet to choose Rules by.", choices=["Implemented", "Recommended"])
    _parser.add_argument("--excel-file", type=str, required=False, default="excel_config_rules.xlsx")
    _parser.add_argument("--param-file", type=str, required=False, default="config_rules_params.json")
    _parser.add_argument("--max-execution-frequency", type=str, required=False, default="TwentyFour_Hours", choices=["One_Hour", "Three_Hours", "Six_Hours", "Twelve_Hours", "TwentyFour_Hours"])
    return _parser
    
def _namify(name) -> str:
    """
        Convert dashed name to camel cased
        Example: access-keys-rotated -> rAccessKeysRotated
    """
    return f"r{''.join([n.capitalize() for n in name.split('-')])}"
    
    
def get_config_rule_dict(_excel_dict, _ou, _config_rule_option) -> list:
    """
        Extracts relevant data from excel document
    """
    _config_rule_list = []
    for column_name in _excel_dict.keys():
        if column_name.strip() == _ou:
            for key, value in _excel_dict[column_name].items():
                if value.lower().strip() == _config_rule_option:
                    _config_rule_list.append(key)
            break
                
    return _config_rule_list
    
def load_json(json_path) -> dict:
    """
        Loads json file into dict object
    """
    with open(f"{os.path.dirname(__file__)}/{json_path}", "r") as j:
        dict_obj = json.load(j)
    j.close()
    
    return dict_obj


def main() -> None:
    """
        Generates a CloudFormation template per OU for AWS Config Rules
    """
    # parse command line
    parser = _init_parser()
    args, _ = parser.parse_known_args()
    ou = args.ou
    config_rule_option = args.config_rule_option.lower()
    excel_sheet_path = args.excel_file
    config_rules_params_path = args.param_file
    max_execution_frequency = args.max_execution_frequency
    
    # read excel document and instatiate ConfigRulesTemplate class
    config_rules_params = load_json(config_rules_params_path)
    excel_data = pd.read_excel(f"{os.path.dirname(__file__)}/{excel_sheet_path}", index_col=0)
    config_rule_list = get_config_rule_dict(excel_data.to_dict(), ou, config_rule_option)
    cf_template = ConfigRulesTemplate(ou, config_rule_list, max_execution_frequency)
    
    # generate CloudFormation template
    for config_rule in cf_template.config_rules:
        cf_template.add_rule_resource(config_rule, config_rules_params)
    with open(f"{os.path.dirname(__file__)}/{cf_template.template_name}", "w+") as t:
        t.write(cf_template.to_yaml())
    t.close()


if __name__ == "__main__":
    main()